package de.hft.stuttgart.citydoctor2.connect.edge;



import de.hft.stuttgart.citydoctor2.connect.edge.CppReferenceHandlingBase;
import de.hft.stuttgart.citydoctor2.connect.edge.NativePointerCastException;

public class AutoProSettingsExport extends CppReferenceHandlingBase {
	private native long createCppClass(

				boolean exportMesh,
				String  meshFileName,
				String  meshFilePath,

				boolean exportIV,
				String  ivFileName,
				String  ivFilePath,

				boolean exportSTEP,
				String  stepFileName,
				String  stepFilePath,

				boolean exportCityGML,
				String  cityGMLFileName,
				String  cityGMLFilePath,
				
				boolean exportValidBuildingsOnly,
				boolean moveBuildingsToOrigin,
				boolean exportIntermediateStagesComplete,
				boolean oneSolidPerBuilding,
				
				double minDistToOrigin_X,
				double minDistToOrigin_Y,
				double minDistToOrigin_Z
				
				
			)
			throws OutOfMemoryError;

	private native void disposeCppClass(long ptrToCppAutoPro)
			throws NativePointerCastException;


	private boolean exportMesh;
	private String  meshFileName;
	private String  meshFilePath;

	private boolean exportIV;
	private String  ivFileName;
	private String  ivFilePath;	

	private boolean exportSTEP;
	private String  stepFileName;
	private String  stepFilePath;

	private boolean exportCityGML;
	private String  cityGMLFileName;
	private String  cityGMLFilePath;
	
	private boolean exportValidBuildingsOnly;
	private boolean moveBuildingsToOrigin;
	private boolean exportIntermediateStagesComplete;
	private boolean oneSolidPerBuilding;
	
	double minDistToOrigin_X;
	double minDistToOrigin_Y;
	double minDistToOrigin_Z;

	public AutoProSettingsExport() {

		setExportMesh(false);
		setMeshFileName("");
		setMeshFilePath("");

		setExportIV(false);
		setIVFileName("");
		setIVFilePath("");

		setExportSTEP(false);
		setSTEPFileName("");
		setSTEPFilePath("");

		setExportCityGML(false);
		setCityGMLFileName("");
		setCityGMLFilePath("");	
		
		setExportValidBuildingsOnly(false);
		setMoveBuildingsToOrigin(false);
		setExportIntermediateStagesComplete(false);
		setExportStepWithOneSolidPerBuilding(false);

		setMinDistToOrigin_X(0);
		setMinDistToOrigin_Y(0);
		setMinDistToOrigin_Z(0);

	}

	/**
	 * Creates an C++ feature object
	 * 
	 * @throws IllegalStateException
	 *             Thrown if the encapsulated "long-Pointer" isn't 0
	 * @throws NativeOutOfMemoryException
	 *             If there is no more memory on the native side
	 * 
	 * @see {@link disposeCppObject}
	 * @see de.hft.stuttgart.citydoctor.connect.edGe.CppReferenceHandlingBase#createCppObject()
	 */
	public void createCppObject() throws IllegalStateException,
			OutOfMemoryError {
		if (0L != ptrToCppObject) {
			throw new IllegalStateException(
					"Can't create new C++ object, there is allready one!");
		}
		// Maybe an exception is thrown
		long tmpPointer = createCppClass(
				isExportMesh(),
				getMeshFileName(),
				getMeshFilePath(),
				isExportIV(),
				getIVFileName(),
				getIVFilePath(),
				isExportSTEP(),
				getSTEPFileName(),
				getSTEPFilePath(),
				isExportCityGML(),
				getCityGMLFileName(),
				getCityGMLFilePath(),
				getExportValidBuildingsOnly(),
				getMoveBuildingsToOrigin(),
				getExportIntermediateStagesComplete(),
				getExportStepWithOneSolidPerBuilding(),
				getMinDistToOrigin_X(),
				getMinDistToOrigin_Y(),
				getMinDistToOrigin_Z()
				);
		ptrToCppObject = tmpPointer;
	}

	@Override
	public void disposeCppObject() throws NativePointerCastException {
		if (0L != ptrToCppObject) {
			disposeCppClass(ptrToCppObject);
			ptrToCppObject = 0L;
		}
	}


	public void setExportMesh(boolean exportMesh) {
		this.exportMesh = exportMesh;
	}

	public void setMeshFileName(String meshFileName) {
		this.meshFileName = meshFileName;
	}

	public void setMeshFilePath (String meshFilePath) {
		this.meshFilePath = meshFilePath;
	}

	public void setExportIV(boolean exportIV) {
		this.exportIV = exportIV;
	}

	public void setIVFileName(String ivFileName) {
		this.ivFileName = ivFileName;
	}

	public void setIVFilePath (String ivFilePath) {
		this.ivFilePath = ivFilePath;
	}

	public void setExportSTEP(boolean exportSTEP) {
		this.exportSTEP = exportSTEP;
	}

	public void setSTEPFileName(String stepFileName) {
		this.stepFileName = stepFileName;
	}

	public void setSTEPFilePath (String stepFilePath) {
		this.stepFilePath = stepFilePath;
	}

	public void setExportCityGML(boolean exportCityGML) {
		this.exportCityGML = exportCityGML;
	}

	public void setCityGMLFileName(String cityGMLFileName) {
		this.cityGMLFileName = cityGMLFileName;
	}

	public void setCityGMLFilePath(String cityGMLFilePath) {
		this.cityGMLFilePath = cityGMLFilePath;
	}
	
	public void setExportValidBuildingsOnly(boolean exportValidBuildingsOnly) {
		this.exportValidBuildingsOnly = exportValidBuildingsOnly;
	}
	
	public void setMoveBuildingsToOrigin(boolean moveBuildingsToOrigin) {
		this.moveBuildingsToOrigin = moveBuildingsToOrigin;
	}
	
	public void setExportIntermediateStagesComplete(boolean exportIntermediateStagesComplete) {
		this.exportIntermediateStagesComplete = exportIntermediateStagesComplete;
	}
	
	public void setExportStepWithOneSolidPerBuilding(boolean oneSolidPerBuilding) {
		this.oneSolidPerBuilding = oneSolidPerBuilding;
	}
	
	public void setMinDistToOrigin_X(double minDistToOrigin_X) {
		this.minDistToOrigin_X = minDistToOrigin_X;
	}
	
	public void setMinDistToOrigin_Y(double minDistToOrigin_Y) {
		this.minDistToOrigin_Y = minDistToOrigin_Y;
	}
	
	public void setMinDistToOrigin_Z(double minDistToOrigin_Z) {
		this.minDistToOrigin_Z = minDistToOrigin_Z;
	}	


	public boolean isExportMesh() {
		return exportMesh;
	}

	public String getMeshFileName() {
		return meshFileName;
	}

	public String getMeshFilePath () {
		return meshFilePath;
	}

	public boolean isExportIV() {
		return exportIV;
	}

	public String getIVFileName() {
		return ivFileName;
	}

	public String getIVFilePath () {
		return ivFilePath;
	}

	public boolean isExportSTEP() {
		return exportSTEP;
	}

	public String getSTEPFileName() {
		return stepFileName;
	}

	public String getSTEPFilePath () {
		return stepFilePath;
	}

	public boolean isExportCityGML() {
		return exportCityGML;
	}

	public String getCityGMLFileName() {
		return cityGMLFileName;
	}

	public String getCityGMLFilePath() {
		return cityGMLFilePath;
	}
	
	public boolean getExportValidBuildingsOnly() {	
		return exportValidBuildingsOnly;
	}
	
	public boolean getMoveBuildingsToOrigin() {	
		return moveBuildingsToOrigin;
	}
	
	public boolean getExportIntermediateStagesComplete() {	
		return exportIntermediateStagesComplete;
	}
	
	public boolean getExportStepWithOneSolidPerBuilding() {
		return oneSolidPerBuilding;
	}
	
	public double getMinDistToOrigin_X() {	
		return minDistToOrigin_X;
	}
	
	public double getMinDistToOrigin_Y() {	
		return minDistToOrigin_Y;
	}
	
	public double getMinDistToOrigin_Z() {	
		return minDistToOrigin_Z;
	}	
}
