package de.hft.stuttgart.citydoctor2.healing;

import org.citygml4j.core.model.construction.AbstractConstructionSurface;
import org.citygml4j.core.model.construction.GroundSurface;
import org.citygml4j.core.model.construction.RoofSurface;
import org.citygml4j.core.model.construction.WallSurface;
import org.citygml4j.core.model.core.AbstractSpaceBoundaryProperty;

import de.hft.stuttgart.citydoctor2.check.HealingID;
import de.hft.stuttgart.citydoctor2.check.HealingMethod;
import de.hft.stuttgart.citydoctor2.check.ModificationListener;
import de.hft.stuttgart.citydoctor2.check.error.PolygonWithoutSurfaceError;
import de.hft.stuttgart.citydoctor2.datastructure.AbstractBuilding;
import de.hft.stuttgart.citydoctor2.datastructure.BoundarySurface;
import de.hft.stuttgart.citydoctor2.datastructure.BoundarySurfaceType;
import de.hft.stuttgart.citydoctor2.datastructure.CityObject;
import de.hft.stuttgart.citydoctor2.datastructure.ConcretePolygon;
import de.hft.stuttgart.citydoctor2.datastructure.Geometry;
import de.hft.stuttgart.citydoctor2.datastructure.GeometryType;
import de.hft.stuttgart.citydoctor2.datastructure.LinkedPolygon;
import de.hft.stuttgart.citydoctor2.datastructure.Polygon;
import de.hft.stuttgart.citydoctor2.math.Vector3d;

public class HealPolygonWithoutSurface implements HealingMethod {

	@Override
	public boolean visit(PolygonWithoutSurfaceError err, ModificationListener l) {
		Polygon polygon = err.getPolygon();
		if (!(polygon instanceof ConcretePolygon cp)) {
			return false;
		}
		CityObject parent = polygon.getParent().getParent();
		if (!(parent instanceof AbstractBuilding ab)) {
			return false;
		}
		Vector3d n = polygon.calculateNormal();
		double tilt = Math.acos(n.getZ() / n.getLength()) * 180 / Math.PI;

		AbstractConstructionSurface acs;
		BoundarySurface bs = new BoundarySurface(null);
		if (tilt < 75.0) {
			acs = new RoofSurface();
			bs.setType(BoundarySurfaceType.ROOF);
		} else if (tilt > 75.0 && tilt < 170.0) {
			acs = new WallSurface();
			bs.setType(BoundarySurfaceType.WALL);
		} else if (tilt > 170.0) {
			acs = new GroundSurface();
			bs.setType(BoundarySurfaceType.GROUND);
		} else {
			return false;
		}
		bs.setGmlObject(acs);
		addBoundarySurfaceToBuilding(cp, ab, bs);
		ab.getGmlObject().getBoundaries().add(new AbstractSpaceBoundaryProperty(acs));
		return true;
	}

	private void addBoundarySurfaceToBuilding(ConcretePolygon cp, AbstractBuilding ab, BoundarySurface bs) {
		Geometry parentGeometry = cp.getParent();
		parentGeometry.getPolygons().remove(cp);
		ab.addBoundarySurface(bs);
		Geometry geom = new Geometry(GeometryType.MULTI_SURFACE, parentGeometry.getLod());
		bs.addGeometry(geom);
		geom.addPolygon(cp);
		parentGeometry.addPolygon(new LinkedPolygon(cp, parentGeometry));
	}

	@Override
	public HealingID getID() {
		return HealingID.SE_POLYGON_WITHOUT_SURFACE;
	}

	@Override
	public HealingMethod createNew() {
		return new HealPolygonWithoutSurface();
	}

}
